package main

import (
	"context"
	"flag"
	"log"
	"os"
	"os/exec"
	"os/signal"
	"time"

	"github.com/creachadair/jrpc2"
	"github.com/creachadair/jrpc2/channel"
)

var (
	cmdPath  = flag.String("command", "./tsrv", "Command path")
	callWait = flag.Duration("wait", 3*time.Second, "Wait before exit")
)

func main() {
	flag.Parse()
	log.Print("cli here")

	cmd := exec.Command(*cmdPath)
	w, err := cmd.StdinPipe()
	if err != nil {
		log.Fatalf("StdinPipe: %v", err)
	}
	r, err := cmd.StdoutPipe()
	if err != nil {
		log.Fatalf("StdoutPipe: %v", err)
	}
	cmd.Stderr = os.Stderr
	if err := cmd.Start(); err != nil {
		log.Fatalf("Start: %v", err)
	}
	log.Printf("Server subprocess started (pid=%d)", cmd.Process.Pid)

	ch := channel.LSP(r, w)
	cli := jrpc2.NewClient(ch, &jrpc2.ClientOptions{
		Logger: jrpc2.StdLogger(log.New(os.Stderr, "[client] ", log.Ldate|log.Ltime|log.Lmicroseconds)),
	})

	ctx, cancel := signal.NotifyContext(context.Background(), os.Interrupt)
	defer cancel()

	select {
	case <-time.After(*callWait):
		if err := cli.Notify(ctx, "exit", nil); err != nil {
			log.Fatalf("Notify: %v", err)
		}
		log.Printf("Subprocess status: %v", cmd.Wait())
	case <-ctx.Done():
		log.Print("<interrupted>")
		return
	}
}
