package main

import (
	"context"
	"flag"
	"log"
	"os"

	"github.com/creachadair/jrpc2"
	"github.com/creachadair/jrpc2/channel"
	"github.com/creachadair/jrpc2/handler"
)

func main() {
	flag.Parse()
	log.Print("srv here")

	ch := channel.LSP(os.Stdin, os.Stdout)
	srv := jrpc2.NewServer(handler.Map{
		"exit": handler.New(func(ctx context.Context) error {
			log.Printf("Exit called")
			jrpc2.ServerFromContext(ctx).Stop()
			return nil
		}),
	}, &jrpc2.ServerOptions{
		Logger: jrpc2.StdLogger(log.New(os.Stderr, "[server] ", log.Ldate|log.Ltime|log.Lmicroseconds)),
	}).Start(ch)
	log.Print("Server started...")

	log.Printf("WaitStatus: %v", srv.WaitStatus())
}
